'use strict';

var vue = require('vue');
var _ = require('../_util/_.js');
var SelectTrigger = require('./SelectTrigger.js');
var OptionList = require('./OptionList.js');
var index = require('./Selector/index.js');
var useSelectTriggerControl = require('./hooks/useSelectTriggerControl.js');
require('../icon/index.js');
var utils = require('./utils.js');
var Icon = require('../icon/Icon.js');

const prefixCls = "x-select";
var Select = vue.defineComponent({
  name: prefixCls,
  props: {
    value: [String, Array],
    options: Array,
    multiple: Boolean,
    disabled: Boolean,
    showArrow: {
      default: true
    },
    clearable: Boolean,
    virtualScroll: Boolean,
    placeholder: {
      default: "\u8BF7\u9009\u62E9"
    },
    filterable: Boolean,
    filter: Function
  },
  slots: ["default", "label"],
  emits: ["update:value", "change", "select", "deselect", "search", "focus", "blur"],
  setup(props, {
    emit,
    expose,
    slots
  }) {
    const containerRef = vue.ref();
    const popupWidth = vue.ref(0);
    const inputRef = vue.ref();
    const popupRef = vue.ref();
    const optionsRef = vue.computed(() => {
      var _a;
      const searchVal = state.searchValue;
      if (props.filterable && searchVal) {
        const filter = props.filter || ((str, option) => utils.getLabel(option).includes(str));
        return (_a = props.options) == null ? void 0 : _a.filter((opt) => filter(searchVal, opt));
      }
      return props.options;
    });
    const optionsBy = vue.computed(() => _.keyBy(props.options, "value"));
    const state = vue.reactive({
      selected: value2Selected(),
      searchValue: ""
    });
    vue.watch(() => props.value, () => {
      state.selected = value2Selected();
    });
    const valuesSet = vue.computed(() => new Set(state.selected));
    const labelValues = vue.computed(() => state.selected.map((e) => optionsBy.value[e]));
    function value2Selected() {
      const {
        value
      } = props;
      return Array.isArray(value) ? value : value != null ? [value] : [];
    }
    function setValue(arr) {
      const value = props.multiple ? [...arr] : arr[0];
      emit("update:value", value);
      emit("change", value);
    }
    function setSearchValue(text) {
      state.searchValue = text;
      emit("search", text);
    }
    const opened = vue.ref(false);
    function onToggleOpen(boo) {
      if (props.disabled)
        return;
      if (boo === void 0) {
        opened.value = !opened.value;
      } else {
        opened.value = boo;
      }
    }
    useSelectTriggerControl([containerRef, popupRef], onToggleOpen);
    function onSelect(option) {
      const value = option.value;
      if (!props.multiple) {
        state.selected = [value];
        emit("select", option.value, option);
      } else {
        const index = state.selected.indexOf(option.value);
        if (~index) {
          state.selected.splice(index, 1);
          emit("deselect", value, option);
        } else {
          state.selected.push(value);
          emit("select", option.value, option);
        }
      }
      state.searchValue = "";
      let values = state.selected;
      values = props.multiple ? [...values] : values[0];
      emit("update:value", values);
      emit("change", value, option);
      if (!props.multiple) {
        onToggleOpen(false);
      }
    }
    function onClear() {
      setValue([]);
      setSearchValue("");
    }
    function onSearch(text) {
      setSearchValue(text);
    }
    function onMenuMousedown(e) {
      e.preventDefault();
    }
    const focusRef = vue.ref(false);
    function onFocus(e) {
      if (props.disabled)
        return;
      focusRef.value = true;
      setSearchValue("");
      emit("focus", e);
    }
    function onBlur(e) {
      focusRef.value = false;
      if (props.disabled)
        return;
      setSearchValue("");
      emit("blur", e);
    }
    function focus() {
      inputRef.value.focus();
    }
    function blur() {
      inputRef.value.blur();
    }
    expose({
      focus,
      blur
    });
    vue.watchPostEffect(() => {
      if (!opened.value)
        return;
      if (!containerRef.value)
        return;
      popupWidth.value = containerRef.value.offsetWidth;
    });
    return () => {
      const {
        clearable,
        showArrow,
        disabled
      } = props;
      const values = state.selected;
      const popup = vue.createVNode(OptionList["default"], {
        "prefixCls": prefixCls,
        "ref": popupRef,
        "values": valuesSet.value,
        "options": optionsRef.value,
        "virtualScroll": props.virtualScroll,
        "onSelect": onSelect,
        "onMousedown": onMenuMousedown
      }, {
        label: slots.label
      });
      const triggerProps = {
        prefixCls,
        popup,
        popupStyle: `width: ${popupWidth.value}px`,
        visible: opened.value,
        disabled: props.disabled,
        filterable: props.filterable
      };
      const cls = [prefixCls, {
        [`${prefixCls}-multiple`]: props.multiple,
        [`${prefixCls}-single`]: !props.multiple,
        [`${prefixCls}-filterable`]: props.filterable,
        [`${prefixCls}-focus`]: focusRef.value,
        [`${prefixCls}-disabled`]: disabled
      }];
      const clearNode = !disabled && clearable && values.length ? vue.createVNode(Icon["default"], {
        "class": `${prefixCls}-clear`,
        "onClick": onClear,
        "type": "cross"
      }, null) : void 0;
      const arrowNode = !disabled && !clearNode && showArrow ? vue.createVNode(Icon["default"], {
        "class": `${prefixCls}-arrow`,
        "type": "arrow-down"
      }, null) : void 0;
      return vue.createVNode("div", {
        "ref": containerRef,
        "class": cls
      }, [vue.createVNode(SelectTrigger, triggerProps, {
        default: () => [vue.createVNode(index, {
          "prefixCls": prefixCls,
          "values": labelValues.value,
          "multiple": props.multiple,
          "searchValue": state.searchValue,
          "filterable": props.filterable,
          "showArrow": props.showArrow,
          "clearable": props.clearable,
          "disabled": props.disabled,
          "placeholder": props.placeholder,
          "inputRef": inputRef,
          "onSelect": onSelect,
          "onSearch": onSearch,
          "onToggleOpen": onToggleOpen,
          "onFocus": onFocus,
          "onBlur": onBlur
        }, null)]
      }), clearNode, arrowNode]);
    };
  }
});

module.exports = Select;
