'use strict';

var vue = require('vue');
var useMemo = require('../_util/hooks/useMemo.js');
var util = require('../align/util.js');
var index = require('../_util/props-util/index.js');

function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !vue.isVNode(s);
}
const prefixCls = "x-virtual-list";
const EMPTY_DATA = Object.freeze([]);
const VirtualList = vue.defineComponent({
  name: prefixCls,
  props: {
    data: Array,
    virtual: Boolean,
    height: {
      default: 256
    },
    itemKey: [String, Number, Function],
    itemsWrap: [Object, Function],
    onScroll: Function
  },
  emits: ["scroll"],
  setup(props, {
    expose,
    emit
  }) {
    const firstNode = vue.ref(), h = vue.ref(0);
    let sizeMonitor;
    const saveFirstNode = (dom) => {
      if (firstNode.value === dom)
        return;
      firstNode.value = dom;
      sizeMonitor == null ? void 0 : sizeMonitor();
      sizeMonitor = util.monitorResize(firstNode.value, () => {
        const {
          marginTop,
          marginBottom
        } = window.getComputedStyle(firstNode.value);
        h.value = dom.offsetHeight + parseFloat(marginTop) + parseFloat(marginBottom);
      });
    };
    const itemHeight = useMemo(() => h.value);
    vue.onUnmounted(() => {
      sizeMonitor == null ? void 0 : sizeMonitor();
    });
    const dataRef = vue.computed(() => props.data || EMPTY_DATA);
    const scrollHeight = vue.computed(() => dataRef.value.length * itemHeight.value);
    const containerRef = vue.ref();
    const barRef = vue.ref();
    const calState = vue.reactive({
      si: 0,
      ei: dataRef.value.length
    });
    const showList = vue.computed(() => dataRef.value.slice(calState.si, calState.ei));
    const state = vue.reactive({
      scrollTop: 0
    });
    function getIndex(index) {
      return calState.si + index;
    }
    function getKey(item, index) {
      var _a;
      if (typeof props.itemKey === "function")
        return props.itemKey(item);
      return (_a = item[props.itemKey]) != null ? _a : index;
    }
    vue.watch([() => state.scrollTop, () => props.virtual, dataRef, () => props.height, () => itemHeight.value], () => {
      if (!props.virtual) {
        calState.si = 0;
        calState.ei = dataRef.value.length;
        return;
      }
      const {
        scrollTop
      } = state;
      let si = ~~(scrollTop / itemHeight.value), ei = Math.ceil((scrollTop + props.height) / itemHeight.value);
      calState.si = si;
      calState.ei = itemHeight.value === 0 ? si + 1 : ei;
    }, {
      immediate: true
    });
    function syncSCroll(top) {
      if (state.scrollTop == top)
        return;
      top = Math.min(top, scrollHeight.value - props.height);
      if (top < 0)
        top = 0;
      containerRef.value.scrollTop = top;
      state.scrollTop = top;
      if (containerRef.value)
        containerRef.value.scrollTop = top;
    }
    function onScroll(e) {
      const {
        scrollTop
      } = e.currentTarget;
      syncSCroll(scrollTop);
      emit("scroll", e);
    }
    function scrollTo(arg) {
      if (arg === void 0)
        return;
      if (typeof arg === "number") {
        syncSCroll(arg);
      } else if (typeof arg === "object") {
        let index = 0;
        if (typeof arg.index === "number") {
          index = arg.index;
        } else {
          index = dataRef.value.findIndex((e, i) => getKey(e, i) === arg.key);
        }
        if (index === void 0)
          return;
        syncSCroll(index * itemHeight.value - (props.height - itemHeight.value >> 1));
      }
    }
    expose({
      scrollTo
    });
    return (vm) => {
      const {
        virtual,
        height,
        itemsWrap
      } = props;
      const comStyle = virtual ? `overflow-y:overlay; overflow-anchor:none;` : "overflow:overlay;";
      const childRender = (item, index$1) => index.getSlot(vm, "default", {
        data: showList.value,
        item,
        index: getIndex(index$1)
      })[0];
      const itemsNode = showList.value.map((item, index) => vue.cloneVNode(childRender(item, index), {
        key: getKey(item, getIndex(index)),
        class: `${prefixCls}_item`,
        ref: index == 0 ? saveFirstNode : void 0
      }));
      let wrapNode = itemsWrap ? vue.createVNode(itemsWrap, null, _isSlot(itemsNode) ? itemsNode : {
        default: () => [itemsNode]
      }) : vue.createVNode("div", {
        "style": "display:flex; flex-direction:column;"
      }, [itemsNode]);
      wrapNode = vue.cloneVNode(wrapNode, {
        style: `transform:translateY(${calState.si * itemHeight.value}px);`
      });
      return (
        // <div class={prefixCls}>
        vue.createVNode("div", {
          "ref": containerRef,
          "class": `${prefixCls}_wrapper`,
          "style": `max-height:${height}px;` + comStyle,
          "onScroll": onScroll
        }, [props.virtual ? vue.createVNode("div", {
          "style": `height:${scrollHeight.value || height}px; position:relative;`
        }, [wrapNode]) : vue.createVNode("div", {
          "style": "position:relative"
        }, [wrapNode])])
      );
    };
  }
});
VirtualList.install = (app) => {
  app.component(VirtualList.name, VirtualList);
};

module.exports = VirtualList;
