import { defineComponent, ref, computed, reactive, watch, watchPostEffect, createVNode } from 'vue';
import { keyBy } from '../_util/_.js';
import SelectTrigger from './SelectTrigger.js';
import OptionList from './OptionList.js';
import Selector from './Selector/index.js';
import useSelectTriggerControl from './hooks/useSelectTriggerControl.js';
import '../icon/index.js';
import { getLabel } from './utils.js';
import Icon from '../icon/Icon.js';

const prefixCls = "x-select";
var Select = defineComponent({
  name: prefixCls,
  props: {
    value: [String, Array],
    options: Array,
    multiple: Boolean,
    disabled: Boolean,
    showArrow: {
      default: true
    },
    clearable: Boolean,
    virtualScroll: Boolean,
    placeholder: {
      default: "\u8BF7\u9009\u62E9"
    },
    filterable: Boolean,
    filter: Function
  },
  slots: ["default", "label"],
  emits: ["update:value", "change", "select", "deselect", "search", "focus", "blur"],
  setup(props, {
    emit,
    expose,
    slots
  }) {
    const containerRef = ref();
    const popupWidth = ref(0);
    const inputRef = ref();
    const popupRef = ref();
    const optionsRef = computed(() => {
      var _a;
      const searchVal = state.searchValue;
      if (props.filterable && searchVal) {
        const filter = props.filter || ((str, option) => getLabel(option).includes(str));
        return (_a = props.options) == null ? void 0 : _a.filter((opt) => filter(searchVal, opt));
      }
      return props.options;
    });
    const optionsBy = computed(() => keyBy(props.options, "value"));
    const state = reactive({
      selected: value2Selected(),
      searchValue: ""
    });
    watch(() => props.value, () => {
      state.selected = value2Selected();
    });
    const valuesSet = computed(() => new Set(state.selected));
    const labelValues = computed(() => state.selected.map((e) => optionsBy.value[e]));
    function value2Selected() {
      const {
        value
      } = props;
      return Array.isArray(value) ? value : value != null ? [value] : [];
    }
    function setValue(arr) {
      const value = props.multiple ? [...arr] : arr[0];
      emit("update:value", value);
      emit("change", value);
    }
    function setSearchValue(text) {
      state.searchValue = text;
      emit("search", text);
    }
    const opened = ref(false);
    function onToggleOpen(boo) {
      if (props.disabled)
        return;
      if (boo === void 0) {
        opened.value = !opened.value;
      } else {
        opened.value = boo;
      }
    }
    useSelectTriggerControl([containerRef, popupRef], onToggleOpen);
    function onSelect(option) {
      const value = option.value;
      if (!props.multiple) {
        state.selected = [value];
        emit("select", option.value, option);
      } else {
        const index = state.selected.indexOf(option.value);
        if (~index) {
          state.selected.splice(index, 1);
          emit("deselect", value, option);
        } else {
          state.selected.push(value);
          emit("select", option.value, option);
        }
      }
      state.searchValue = "";
      let values = state.selected;
      values = props.multiple ? [...values] : values[0];
      emit("update:value", values);
      emit("change", value, option);
      if (!props.multiple) {
        onToggleOpen(false);
      }
    }
    function onClear() {
      setValue([]);
      setSearchValue("");
    }
    function onSearch(text) {
      setSearchValue(text);
    }
    function onMenuMousedown(e) {
      e.preventDefault();
    }
    const focusRef = ref(false);
    function onFocus(e) {
      if (props.disabled)
        return;
      focusRef.value = true;
      setSearchValue("");
      emit("focus", e);
    }
    function onBlur(e) {
      focusRef.value = false;
      if (props.disabled)
        return;
      setSearchValue("");
      emit("blur", e);
    }
    function focus() {
      inputRef.value.focus();
    }
    function blur() {
      inputRef.value.blur();
    }
    expose({
      focus,
      blur
    });
    watchPostEffect(() => {
      if (!opened.value)
        return;
      if (!containerRef.value)
        return;
      popupWidth.value = containerRef.value.offsetWidth;
    });
    return () => {
      const {
        clearable,
        showArrow,
        disabled
      } = props;
      const values = state.selected;
      const popup = createVNode(OptionList, {
        "prefixCls": prefixCls,
        "ref": popupRef,
        "values": valuesSet.value,
        "options": optionsRef.value,
        "virtualScroll": props.virtualScroll,
        "onSelect": onSelect,
        "onMousedown": onMenuMousedown
      }, {
        label: slots.label
      });
      const triggerProps = {
        prefixCls,
        popup,
        popupStyle: `width: ${popupWidth.value}px`,
        visible: opened.value,
        disabled: props.disabled,
        filterable: props.filterable
      };
      const cls = [prefixCls, {
        [`${prefixCls}-multiple`]: props.multiple,
        [`${prefixCls}-single`]: !props.multiple,
        [`${prefixCls}-filterable`]: props.filterable,
        [`${prefixCls}-focus`]: focusRef.value,
        [`${prefixCls}-disabled`]: disabled
      }];
      const clearNode = !disabled && clearable && values.length ? createVNode(Icon, {
        "class": `${prefixCls}-clear`,
        "onClick": onClear,
        "type": "cross"
      }, null) : void 0;
      const arrowNode = !disabled && !clearNode && showArrow ? createVNode(Icon, {
        "class": `${prefixCls}-arrow`,
        "type": "arrow-down"
      }, null) : void 0;
      return createVNode("div", {
        "ref": containerRef,
        "class": cls
      }, [createVNode(SelectTrigger, triggerProps, {
        default: () => [createVNode(Selector, {
          "prefixCls": prefixCls,
          "values": labelValues.value,
          "multiple": props.multiple,
          "searchValue": state.searchValue,
          "filterable": props.filterable,
          "showArrow": props.showArrow,
          "clearable": props.clearable,
          "disabled": props.disabled,
          "placeholder": props.placeholder,
          "inputRef": inputRef,
          "onSelect": onSelect,
          "onSearch": onSearch,
          "onToggleOpen": onToggleOpen,
          "onFocus": onFocus,
          "onBlur": onBlur
        }, null)]
      }), clearNode, arrowNode]);
    };
  }
});

export { Select as default };
